#include "../../include/all.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define DATA_DIR "/tmp/data"
#define TEMP_DIR "/tmp/temp"
#define ROOT_DIR "/tmp/root"

int main(int argc, char *argv[]) {
  int       ret = EXIT_FAILURE;
  lm_pkg_t *pkg = NULL;
  lm_ctx_t  ctx;

  if (argc != 2) {
    printf("usage: %s <pool url>\n", argv[0]);
    return ret;
  }

  if (!mkdir_ifnot(ROOT_DIR)) {
    printf("failed to create root dir: %s", ROOT_DIR);
    return ret;
  }

  lm_ctx_init(&ctx);

  if (!lm_ctx_set_data(&ctx, DATA_DIR)) {
    printf("failed to set data dir: %s (%d)\n", lm_strerror(), lm_error());
    goto end;
  }

  if (!lm_ctx_set_temp(&ctx, TEMP_DIR)) {
    printf("failed to set temp dir: %s (%d)\n", lm_strerror(), lm_error());
    goto end;
  }

  if (!lm_ctx_set_root(&ctx, ROOT_DIR)) {
    printf("failed to set root dir: %s (%d)\n", lm_strerror(), lm_error());
    goto end;
  }

  if (lm_ctx_pool_add(&ctx, "base", argv[1]) == NULL) {
    printf("failed to add pool: %s (%d)\n", lm_strerror(), lm_error());
    goto end;
  }

  lm_ctx_ping(&ctx, NULL, NULL);
  lm_ctx_sync(&ctx, true, NULL, NULL);

  if ((pkg = lm_ctx_find(&ctx, "which", NULL)) == NULL) {
    printf("failed to find package: %s (%d)\n", lm_strerror(), lm_error());
    goto end;
  }

  if (!lm_ctx_download(&ctx, pkg, NULL, NULL)) {
    printf("failed to download package: %s (%d)\n", lm_strerror(), lm_error());
    goto end;
  }

  if (!lm_ctx_install(&ctx, pkg, NULL, NULL)) {
    printf("failed to install the package: %s (%d)\n", lm_strerror(), lm_error());
    goto end;
  }

  ret = EXIT_SUCCESS;

end:
  lm_ctx_free(&ctx);
  return ret;
}
