#include "../../include/error.h"
#include "../../include/util.h"
#include "../../include/ctx.h"

#include <linux/limits.h>
#include <stdbool.h>
#include <stdlib.h>
#include <locale.h>
#include <string.h>
#include <stdio.h>

void lm_ctx_init(lm_ctx_t *ctx) {
  setlocale(LC_ALL, "");
  textdomain("libmp");

  bzero(ctx, sizeof(lm_ctx_t));
  ctx->version = LM_VERSION;
  
  lm_error_clear();
}

bool lm_ctx_set_temp(lm_ctx_t *ctx, char *dir){
  if(!mkdir_ifnot(dir)){
    lm_error_set(LM_ERR_CtxTempFailMkdir);
    return false;
  }

  if(!is_dir(dir)){
    lm_error_set(LM_ERR_CtxTempNotDir);
    return false;
  }

  if(!can_write(dir)){
    lm_error_set(LM_ERR_CtxTempNoWrite);
    return false;
  }

  char fullpath[PATH_MAX + 1];
  realpath(dir, fullpath);

  ctx->temp = strdup(fullpath);
  return true;
}

bool lm_ctx_set_root(lm_ctx_t *ctx, char *dir){
  if(!exists(dir)){
    lm_error_set(LM_ERR_CtxRootFail);
    return false;
  }

  if(!is_dir(dir)){
    lm_error_set(LM_ERR_CtxRootNotDir);
    return false;
  }

  if(!can_write(dir)){
    lm_error_set(LM_ERR_CtxRootNoWrite);
    return false;
  }

  char fullpath[PATH_MAX + 1];
  realpath(dir, fullpath);

  ctx->root = strdup(fullpath);
  return true;
}

bool lm_ctx_set_data(lm_ctx_t *ctx, char *dir){
  if(!mkdir_ifnot(dir)){
    lm_error_set(LM_ERR_CtxDataFailMkdir);
    return false;
  }

  if(!is_dir(dir)){
    lm_error_set(LM_ERR_CtxDataNotDir);
    return false;
  }

  if(!can_write(dir)){
    lm_error_set(LM_ERR_CtxDataNoWrite);
    return false;
  }

  char fullpath[PATH_MAX + 1];
  realpath(dir, fullpath);

  ctx->data = strdup(fullpath);
  return true;
}

void lm_ctx_free(lm_ctx_t *ctx) {
  lm_ctx_pool_clear(ctx);
  free(ctx->data);
  free(ctx->root);
  free(ctx->temp);

  if(NULL != ctx->db)
    lm_database_free(ctx->db);
  
  lm_error_clear();
  return;
}
