#include "../../include/database.h"
#include "../../include/error.h"
#include "../../include/util.h"
#include "../../include/ctx.h"

#include <string.h>

bool lm_ctx_database_init(lm_ctx_t *ctx){
  if(ctx->db != NULL)
    return true;

  char dbpath[strlen(ctx->data)+10];
  join(dbpath, ctx->data, "db");

  ctx->db = lm_database_new(dbpath);
  return ctx->db != NULL; // error set by function
}

bool lm_ctx_database_is_installed(lm_ctx_t *ctx, lm_pkg_t *pkg, bool check_version){
  if(!lm_ctx_database_init(ctx))
    return false; // error set by function
  lm_pkg_t found;

  if(NULL == ctx->db)
    return false;

  if(!lm_database_package_find(ctx->db, &found, pkg->name))
    return false; // error set by function

  if(check_version && !eq(found.version, pkg->version))
    return false;

  return true;
}

bool lm_ctx_database_find(lm_ctx_t *ctx, lm_pkg_t *pkg, char *name){
  if(NULL == ctx || NULL == pkg || NULL == name){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  if(!lm_ctx_database_init(ctx))
    return false; // error set by function
  
  lm_error_clear();
  return lm_database_package_find(ctx->db, pkg, name);
}

bool lm_ctx_database_foreach(lm_ctx_t *ctx, lm_ctx_database_callback_t callback, void *data){
  if(NULL == ctx || NULL == callback){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }
  
  if(!lm_ctx_database_init(ctx))
    return false; // error set by function
  
  lm_pkg_t pkg;

  while(lm_database_package_next(ctx->db, &pkg)){
    if(!callback(ctx, &pkg, data))
      break;
  }

  return true;
}
