#include "../../include/ctx.h"
#include "../../include/pool.h"
#include "../../include/util.h"
#include "../../include/error.h"

#include <stdbool.h>
#include <stdlib.h>
#include <string.h>

lm_pool_t *lm_ctx_pool_add(lm_ctx_t *ctx, char *name, char *url) {
  if(NULL == name){
    lm_error_set(LM_ERR_ArgNULL);
    return NULL;
  }

  lm_pool_t *pool = lm_pool_new(name, url);
  if (NULL == pool)
    return NULL;

  pdebug(__func__, "adding new pool: %s (%s://%s:%d%s)", pool->name, pool->url.protocol, pool->url.host, pool->url.port, pool->url.path);

  if(NULL != ctx->data){
    char pools_dir[strlen(ctx->data)+strlen(pool->name)+10];
    join(pools_dir, ctx->data, "pools");

    if(!mkdir_ifnot(pools_dir)){
      lm_error_set(LM_ERR_CtxDataFailMkdir);
      lm_pool_free(pool);
      return NULL;
    }

    bzero(pools_dir, sizeof(pools_dir));
    join_multiple(pools_dir,  ctx->data, "pools", pool->name);

    if(!lm_pool_path_set_dir(pool, pools_dir)){
      // error is set by set_dir
      lm_pool_free(pool);
      return NULL;
    }
  }

  // add to the start
  pool->next = ctx->pools;
  ctx->pools = pool;

  return pool;
}

void lm_ctx_pool_clear(lm_ctx_t *ctx) {
  lm_pool_t *cur = ctx->pools, *prev = NULL;
  while (NULL != cur) {
    prev = cur;
    cur  = cur->next;
    lm_pool_free(prev);
  }

  ctx->pools = NULL;
}

lm_pool_t *lm_ctx_pool_find(lm_ctx_t *ctx, char *name) {
  lm_pool_t *cur = ctx->pools;
  while (NULL != cur) {
    if (eq(cur->name, name))
      return cur;
    cur = cur->next;
  }
  return NULL;
}

lm_pool_t *lm_ctx_pool_by_url(lm_ctx_t *ctx, char *host, char *path) {
  lm_pool_t *cur = ctx->pools;
  while (NULL != cur) {
    if (eq(cur->url.host, host) && eq(cur->url.path, path))
      return cur;
    cur = cur->next;
  }
  return NULL;
}
