#include "../../include/package.h"
#include "../../include/database.h"
#include "../../include/error.h"
#include "../../include/pool.h"
#include "../../include/util.h"
#include "../../include/ctx.h"

#include <stdlib.h>
#include <string.h>

lm_ctx_update_list_t *lm_ctx_update(lm_ctx_t *ctx){
  lm_ctx_update_list_t *list = malloc(sizeof(lm_ctx_update_list_t));
  list->packages = NULL;
  list->index = 0;
  list->count = 0;

  if(!lm_ctx_database_init(ctx))
    return NULL;

  lm_pkg_t *pkg = malloc(sizeof(lm_pkg_t));

  while(lm_database_package_next(ctx->db, pkg)){
    if(NULL == list->packages)
      list->packages = malloc(sizeof(lm_pkg_t*));
    else
      list->packages = realloc(list->packages, (list->count+1)*sizeof(lm_pkg_t*));

    list->packages[list->count++] = pkg;
    pkg = malloc(sizeof(lm_pkg_t));
  }

  return list;
}

lm_pkg_t *lm_ctx_update_next(lm_ctx_t *ctx, lm_ctx_update_list_t *list){
  if(NULL == ctx || NULL == list){
    lm_error_set(LM_ERR_ArgNULL);
    return NULL;
  }

  if(list->index >= list->count-1){
    list->index = 0;
    return NULL;
  }

  return list->packages[list->index++];
}

void lm_ctx_update_free(lm_ctx_t *ctx, lm_ctx_update_list_t *list){
  if(NULL == ctx || NULL == list){
    lm_error_set(LM_ERR_ArgNULL);
    return;
  }

  for(int i = 0; i < list->count; i++){
    lm_package_free(list->packages[i]);
    free(list->packages[i]);
  }
  
  free(list);
}
