#include "../../include/database.h"
#include "../../include/error.h"
#include "../../include/util.h"

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

bool lm_database_changes_update(lm_database_t *db, lm_pkg_t *pkg, char *file){
  if(NULL == db || NULL == pkg || NULL == file){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  char changes_file[strlen(pkg->name)+20];
  sprintf(changes_file, "%s_changes", pkg->name);

  char changes_path[strlen(db->dir)+sizeof(changes_file)];
  join(changes_path, db->dir, changes_file);

  return copy_file(changes_path, file);
}

bool lm_database_changes_del(lm_database_t *db, lm_pkg_t *pkg){
  if(NULL == db || NULL == pkg){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  char changes_file[strlen(pkg->name)+20];
  sprintf(changes_file, "%s_changes", pkg->name);

  char changes_path[strlen(db->dir)+sizeof(changes_file)];
  join(changes_path, db->dir, changes_file);

  if(unlink(changes_path) < 0 && errno != ENOENT){
    lm_error_set(LM_ERR_DbChangesUnlinkFail);
    return false;
  }
  
  return true;
}

char *lm_database_changes_get(lm_database_t *db, lm_pkg_t *pkg){
  char changes_file[strlen(pkg->name)+20];
  sprintf(changes_file, "%s_changes", pkg->name);

  char *changes_path = malloc(strlen(db->dir)+sizeof(changes_file));
  join(changes_path, db->dir, changes_file);

  if(!exists(changes_path)){
    free(changes_path);
    return NULL;
  }

  return changes_path;
}
